package chim

import (
	"fmt"
	"net/http"
	"time"

	"github.com/go-chi/chi/v5/middleware"

	"a.yandex-team.ru/library/go/core/log"
	"a.yandex-team.ru/library/go/core/log/ctxlog"
)

func LogRequest(l log.Logger) func(http.Handler) http.Handler {
	return func(next http.Handler) http.Handler {
		fn := func(w http.ResponseWriter, r *http.Request) {
			t := time.Now()
			ww := middleware.NewWrapResponseWriter(w, r.ProtoMajor)
			defer func() {
				ctxlog.Info(r.Context(), l,
					fmt.Sprintf("req: %s", r.RequestURI),
					log.String("ip", r.RemoteAddr),
					log.String("uri", r.RequestURI),
					log.Int("status", ww.Status()),
					log.String("elapsed", time.Since(t).String()),
				)
			}()

			next.ServeHTTP(ww, r)
		}
		return http.HandlerFunc(fn)
	}
}
